#include "qtdeployer.h"
#include "./ui_qtdeployer.h"
#include <QFile>
#include <QTextStream>
#include <QDebug>
#include <QFileDialog>
#include <QtCore/QCoreApplication>
#include <QProcess>
#include <QThread>

QtDeployer::QtDeployer(QWidget *parent)
    : QMainWindow(parent)
    , ui(new Ui::QtDeployer)
{
    ui->setupUi(this);
    setWindowIcon(QIcon(":/images/deployer.png"));
    setWindowFlags(Qt::Dialog | Qt::MSWindowsFixedSizeDialogHint); //Данная строчка запрещает изменение размеров окна
    setWindowTitle("QtDeployer 1.5");

    formAbout = new about;
    password = new Password;
    Options  = new options;

    Qt::WindowFlags flags = Qt::Dialog;
    password->setWindowFlags(flags);
    formAbout->setWindowFlags(flags);
    //Отключить главное окно при появлении дочернего окна
    password-> setWindowModality (Qt :: ApplicationModal); // Блокируем все формы, кроме текущей формы
    formAbout->setWindowModality (Qt :: ApplicationModal); // Блокируем все формы, кроме текущей формы
    Options->setWindowModality (Qt :: ApplicationModal);   // Блокируем все формы, кроме текущей формы


    ui->textEdit->setTextColor( QColor( "white" ) );
    ui->tE2->setTextColor( QColor( "white" ) );

    connect(ui->installBtn, SIGNAL(clicked()), this, SLOT(install()));
    connect(ui->fileBtn,    SIGNAL(clicked()), this, SLOT(openFile()));
    connect(ui->makeBtn,    SIGNAL(clicked()), this, SLOT(makeProgram()));
    connect(ui->openDirBtn, SIGNAL(clicked()), this, SLOT(openDir()));
    connect(ui->aboutBtn,   SIGNAL(clicked()), this, SLOT(About()));
    connect(password, &Password::signalPasswd, this, &QtDeployer::slotPasswd); //Данное выражение получает сигнал из второй формы
    connect(ui->optionsBtn, SIGNAL(clicked()), this, SLOT(FormOptions()));
    connect(Options, &options::signalFilePath, this, &QtDeployer::slotOption1);
    connect(Options, &options::signalOpenPath, this, &QtDeployer::slotOption2);
    connect(Options, &options::signalGlibc, this, &QtDeployer::slotOption3);
    connect(this, &QtDeployer::signal,  Options, &options::slot);  //Отправляем сигнал состояния конфига в форму options
    connect(this, &QtDeployer::signal2, Options, &options::slot2);  //Отправляем сигнал состояния конфига в форму options
    connect(this, &QtDeployer::signal3, Options, &options::slot3);  //Отправляем сигнал состояния конфига в форму options


    ui->tE2->setText("Выберете release-файл вашей программы!");
    ui->tE2->setAlignment(Qt::AlignCenter);

    if (!QFile("/usr/local/bin/linuxdeployqt").exists()){
        ui->textEdit->setText("Внимание! Программа linuxdeployqt не установлена, а без нее ничего не заработает! Чтобы решить эту проблему, нажмите на соответствующую кнопку в самом низу данной формы. Установка будет произведена в папку /usr/local/bin, поэтому QtDeployer спросит у вас пароль суперпользователя.");
        ui->textEdit->setAlignment(Qt::AlignJustify);
        this->resize(400, 440);
    }
    else{
        this->resize(400, 340);
    }

    //Следующая конструкция читает путь к файлу, выбранному в прошлый раз из конфигурационного файла path
    QFile file("path");
    if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
    qDebug() << "Ошибка при открытии файла";
    QTextStream in(&file);
    FN=in.readAll(); //
    file.close();
    configCheck();
}

QtDeployer::~QtDeployer()
{
    delete ui;
}

void QtDeployer::install()
{    
    if (QFile("/usr/local/bin/linuxdeployqt").exists()){
      ui->tE2->setText("Программа уже установлена!");
      ui->tE2->setAlignment(Qt::AlignCenter);
    }
    else {
      password->show();
    }
}

void QtDeployer::openFile()
{
    //далее новый вариант открытия файла где можно выбрать тип файла
    ui->textEdit->setTextColor( QColor( "white" ) );
    ui->textEdit->setText(" ");
    QString fileName = QFileDialog::getOpenFileName(this,
                                            QString("Открыть файл"),
                                            QDir::currentPath(),
                                     "Все файлы (*)");
         FN=fileName; //Запись имени файла в глобальную переменную FN
         QFile file(fileName);
         ui->tE2->setText("");
         glibc_check();
         ui->tE2->setText("Все готово к сборке!");
         ui->tE2->setAlignment(Qt::AlignCenter);
         ui->textEdit->setAlignment(Qt::AlignJustify);         
}

void QtDeployer::makeProgram()
{
    if (FN ==""){
        ui->tE2->setText("Release-файл программы не выбран!");
        ui->tE2->setAlignment(Qt::AlignCenter);
    }
    else{
        if (QFile(FN).exists()){

        QString make = ui->textEdit->toPlainText(); //Берем текст команды с textEdit
        QProcess* p = new QProcess();               //Инициируем QProcess p, который выпилнит команду, описанную строчкой выше
        p->start("sh", QStringList() << "-c" << QString(make));
        ui->tE2->setText("Процесс пошел!");
        ui->textEdit->setText("Дождитесь, пока ваша программа соберется. Обычно этот процесс занимает несколько секунд, в зависимости от сложности проекта.");
        ui->textEdit->setAlignment(Qt::AlignJustify);
        ui->tE2->setAlignment(Qt::AlignCenter);
        }
        else {
            ui->textEdit->setText("Файл " + FN + " отсутствует");
            ui->tE2->setText("Выберете release-файл.");
            ui->tE2->setAlignment(Qt::AlignCenter);

        }
    }
    if (openPath == true) {
        QString path = QFileInfo(FN).path();        //Получаем путь к файлу без самого файла
        QString command = "nautilus " + path;       //Формируем команду для открытия папки
        QProcess* nautilus = new QProcess();        //Инициируем QProcess p, который выпилнит команду, описанную строчкой выше
        nautilus->start("sh", QStringList() << "-c" << QString(command));
    }
}

void QtDeployer::simLink()
{
    QString command = ui->tE2->toPlainText();
    QProcess* Process = new QProcess();
    Process->start("sh", QStringList() << "-c" << QString(command));
}

void QtDeployer::About()
{
    formAbout->show();
}

void QtDeployer::glibc_check()
{
        if(!glibcFlag == true)
        {
            ui->textEdit->setText(("linuxdeployqt " + FN));
        }
        else {
            ui->textEdit->setText(("linuxdeployqt " + FN + " -unsupported-allow-new-glibc"));
        }
        ui->textEdit->setAlignment(Qt::AlignJustify);
}

void QtDeployer::FNcheck()
{
    if (FN ==""){
        ui->textEdit->setText("\r\n\r\n\r\nhttps://madmentat.ru");
        QTextCursor Cursor = ui->textEdit->textCursor();
        Cursor.movePosition(QTextCursor::End);
        ui->textEdit->setTextCursor(Cursor);
        ui->textEdit->setAlignment(Qt::AlignCenter);
    }
    else {
        glibc_check();
    }
}

void QtDeployer::FormOptions()
{
    Options->show();
}





void QtDeployer::configCheck()
{

    //Следующая конструкция читает конфиг
    QFile config("config");
    if (!config.open(QIODevice::ReadOnly | QIODevice::Text))
    qDebug() << "Ошибка при открытии файла";
    QTextStream cfg(&config);
    QString str = cfg.readAll();
    config.close();
    if (str.contains("SavePath = YES")){ //Если конфиг содержит SavePath = YES
         pathFlag = true;                //Ставим соответствующий флажок в положение true
         FNcheck();
    }
    else if (str.contains("SavePath = NO")){
         pathFlag = false;               //В противном случае в false
    }

    if (str.contains("OpenPath = YES")){ //Если конфиг содержит SavePath = YES
         openPath = true;                //Ставим соответствующий флажок в положение true
         FNcheck();
    }
    else if (str.contains("OpenPath = NO")){
         openPath = false;               //В противном случае в false
    }
    if (str.contains("glibcFlag = YES")){
        glibcFlag = true;
        FNcheck();

    }
    else if (str.contains("glibcFlag = NO")){
        glibcFlag = false;
        FNcheck();

    }


    emit signal(pathFlag);               //Отправляем pathFlag в форму Options
    emit signal2(openPath);
    emit signal3(glibcFlag);
}

void QtDeployer::openDir()
{
    QString path = QFileInfo(FN).path();        //Получаем путь к файлу без самого файла
    QString command = "nautilus " + path;       //Формируем команду для открытия папки
    QProcess* nautilus = new QProcess();        //Инициируем QProcess p, который выпилнит команду, описанную строчкой выше
    nautilus->start("sh", QStringList() << "-c" << QString(command));
}


void QtDeployer::ConfigWrite()
{
    if   (pathFlag == false){
          pathOption = "SavePath =  NO\r\n";
    }
    else {
          pathOption = "SavePath = YES\r\n";
    }
    QString openPathOption, glibc;
    if   (openPath == false){
          openPathOption = "OpenPath =  NO\r\n";
    }
    else {
          openPathOption = "OpenPath = YES\r\n";
    }

    if   (glibcFlag == false){
          glibc = "glibcFlag = NO\r\n";
    }
    else {
          glibc = "glibcFlag = YES\r\n";
    }


    QString command = pathOption + openPathOption + glibc;
    QFile config("config");
    if (!config.open(QIODevice::WriteOnly | QIODevice::Text))
        qDebug() << "Ошибка при открытии файла";
    QTextStream out(&config);//поток для записи текста
    out << command;
    config.close();
}


void QtDeployer::keyPressEvent(QKeyEvent *event)
{
    int key=event->key();
    QString str = QString(QChar(key));
    if (key>=Qt::Key_0 && key<=Qt::Key_F1) {
    formAbout->show();
    }
}

void QtDeployer::slotPasswd(QString pswd)
{
    //Здесь мы принимаем pswd из окна password
    QString myPath = QDir::currentPath(); //Узнаем директорию, в которой находится наша программа
    PASSWORD = pswd;                      //Приравниваем PASSWRD к pswd
    QString command = "echo " + PASSWORD + "| sudo -S cp " + myPath +"/linuxdeployqt.AppImage /usr/local/bin/linuxdeployqt"; //Формируем команду копирования Appimage в папку /usr/local/bin
    QProcess* p = new QProcess();         //Инициируем QProcess p, который выпилнит команду, описанную строчкой вышеЁ
    p->start("sh", QStringList() << "-c" << QString(command));
    ui->tE2->setText("Готово!");
    ui->tE2->setAlignment(Qt::AlignCenter);
    FNcheck();                            //Инициируем проверку содаржания переменной FN
    this->resize(400, 360);               //Меняем высоту формы, чтобы скрыть кнопку копирования linuxdeployqt
}

void QtDeployer::slotOption1(bool filePath)
{

    pathFlag = filePath;

    if (filePath == true){
        if (FN != ""){
            ui->tE2->setText("Путь к release-файлу сохранен.");

        }
     else if (FN == ""){
            ui->tE2->setText("Путь к release-файлу не выбран.");
            }

    ui->tE2->setAlignment(Qt::AlignCenter);

    QFile file("path");
    if (!file.open(QIODevice::WriteOnly | QIODevice::Text))
             return;
    QTextStream out(&file);//поток для записи текста
    out << FN;

    }
    else {
    ui->tE2->setText("Путь к release-файлу забыт.");
    ui->tE2->setAlignment(Qt::AlignCenter);

    QFile file("path");
    if (!file.open(QIODevice::WriteOnly | QIODevice::Text))
             return;
    QTextStream out(&file);//поток для записи текста
    QString FilePath = "";
    out << FilePath;
    file.close();
    }
    ConfigWrite();
}

void QtDeployer::slotOption2(bool opPath)
{
    openPath = opPath;
    ConfigWrite();
}

void QtDeployer::slotOption3(bool gl)
{
     glibcFlag = gl;
     FNcheck();
     ConfigWrite();
}




